loader.addOnLoad(function() {

	var checker = {
		table: null
	};

	utils.mixin(PARAMS, {
		MAX_BOARD_HEIGHT: 100
	});

	var PLAYER = {
		GLOBAL_CHIP: null,
		EMPTY_RECT: 0,
		WHITE_CHIP: 2,
		BLACK_CHIP: 3,
		WHITE_CHIP_KING: 4,
		BLACK_CHIP_KING: 5,
		PERMITTED_POSITION: -1,
		getBoardChipClass : function(x, y) {
			return (((x + y) % 2) == 0 ? "fillRect" : "");
		},
		generateBoardIsCompleted : function(boardNode) {
			boardNode.addEventListener('click', function(evt) {
				var node = evt.target;
				if (node && node.tagName == "DIV") {
					PLAYER.onBoardTdClick(node);
				}
				if (node && node.tagName == "SPAN" && utils.hasClass(node, 'boardContent')) {
					PLAYER.onChipClick(node);
				}
			});
		},
		onChipClick: function(node) {
			PLAYER.GLOBAL_CHIP = node;
		},
		onBoardTdClick: function (node) {
			var x = node.getAttribute('x');
			var y = node.getAttribute('y');
			if (PLAYER.GLOBAL_CHIP) {
				var pNode = PLAYER.GLOBAL_CHIP.parentNode;
				var xfrom = pNode.getAttribute('x');
				var yfrom = pNode.getAttribute('y');
				makeMove(x, y, xfrom, yfrom);
				PLAYER.GLOBAL_CHIP = null;
			}
		},
		getParamsForMove: function(xt, yt, fx, fy) {
			return ({
				xfrom: fx,
				yfrom: fy,
				y: xt,
				x: yt
			});
		}
	};

	// generate html Board
	// Checker and Reversi
	function generateStringHTMLCheckerBoard(node) {
		var result = [];
		result.push("<div id='boardTable' class='boardContainer'>");
		var index = 0;
		for (var i = 0; i < 8; ++i) {
			result.push("<div class='boardRow'>");
			for (var j = 0; j < 8; ++j) {
				var _c = PLAYER.getBoardChipClass(i, j);
				result.push("<div index='" + index + "' id='" + generateChipId(index) + "' y='" + i + "' x='" + j + "' class='" + _c + "'></div>");
				index += 1;
			}
			result.push("</div>");
		}
		result.push("</div>");
		node.innerHTML = result.join("");
		var table = $('boardTable');
		PLAYER.generateBoardIsCompleted(table);
		checker.table = table;
	}

	function generateChipId(index) {
		return (["chip", index].join("_"));
	}

	function onChangeBoardSize(w, h) {
		var node = checker.table;
		var setH = Math.min(w, h) - 10;
		if (setH < PARAMS.MAX_BOARD_HEIGHT) {
			setH = PARAMS.MAX_BOARD_HEIGHT
		}
		node.style.height = setH + 'px';
		node.style.width = setH + 'px';
		var tdH = setH / 8 - 3;
		var tds = $$(node, "div");
		for (var i = 0, l = tds.length; i < l; ++i) {
			var td = tds[i];
			if (td.getAttribute("x")) {
				td.style.width = tdH + 1 + "px";
				td.style.height = tdH + 1 + "px";
				td.style.backgroundSize = tdH + "px";
			}
		}
	}

	function resizeBoard() {
		var node = $("board");
		onChangeBoardSize(node.clientWidth, node.clientHeight)
	}

	function placeChipsToBoard(boardChips, moveSide) {
		for (var i = 0; i < boardChips.length; ++i) {
			var chipState = boardChips[i];
			var chipNode = $("chip_" + i);
			var className = "";
			switch (chipState) {
				case PLAYER.WHITE_CHIP:
					className = "whiteChip";
					break;
				case PLAYER.BLACK_CHIP:
					className = "blackChip";
					break;
				case PLAYER.WHITE_CHIP_KING:
					className = "whiteChipKing";
					break;
				case PLAYER.BLACK_CHIP_KING:
					className = "blackChipKing";
					break;
				case PLAYER.PERMITTED_POSITION:
					if (moveSide == "0") {
						className = "PermittedPosition";
					}
					break;
			}
			utils.addClass(chipNode, className);
		}
	}

	function removeClasses(className) {
		var boardNode = $('boardTable');
		var nodes = boardNode.getElementsByClassName(className);
		while (nodes.length > 0) {
			utils.removeClass(nodes[0], className);
		}
	}

	function selectChip(row, col) {
		if (row != null && col != null) {
			var id = generateChipId(row * 8 + col);
			var node = $(id);
			utils.mobileClick(node, function() {
				utils.addClass(node, "selectChip");
			});
		}
	}

	function reloadBoard(boardChips, moveSide) {
		utils.forEach(["selectChip", "PermittedPosition", "blackChip", "whiteChip", "blackChipKing", "whiteChipKing"], removeClasses);
		placeChipsToBoard(boardChips, moveSide);
	}

	window.addEventListener("resize", resizeBoard);
	window.addEventListener("orientationchange", resizeBoard);

	(function() {
		var boardNode = $("boardTableContainer");
		generateStringHTMLCheckerBoard(boardNode);
		resizeBoard();
	})();

	//!====================================
	//!Algorithm from https://github.com/dvander/checkers
	var Checkers = {
		PlayerOne: 2,
		PlayerTwo: 3,

		MoveList: function (size) {
			this.list = new Array(size);
			this.length = 0;

			this.add = function (fromRow, fromCol, toRow, toCol) {
				var b = (fromRow << 24) |
						(fromCol << 16) |
						(toRow << 8) |
						(toCol);
				this.list[this.length++] = b;
			};

			this.clear = function () {
				this.length = 0;
			};
		},

		Game: function (ui) {
			var PlayerOne = 2;
			var PlayerTwo = 3;
			var rows = ui.rows;
			var cols = ui.cols;
			var RowsPerPlayer = (rows - 2) >> 1;
			var BoardArea = rows * cols;
			var PiecesPerPlayer = RowsPerPlayer * (cols / 2);

			var Board = function () {
				this.rows = rows;
				this.cols = cols;
				this.pieceCount = [0, 0, PiecesPerPlayer, PiecesPerPlayer];

				this.grid = new Array(BoardArea + (PiecesPerPlayer * 2));
			};

			Board.prototype.initPiece = function (row, col, key, player) {
				this.grid[row * this.cols + col] = (key << 8) | player;
				this.grid[BoardArea + key] = (row << 8) | col;
			};

			Board.prototype.removePiece = function (row, col) {
				var pos = row * this.cols + col;
				var piece = this.grid[pos];
				this.grid[pos] = 0;

				var key = (piece >> 8) & 0xFF;
				var player = piece & 0x3;
				this.grid[BoardArea + key] = -1;

				// If removing a king, decrement the king count.
				if (piece & 4) {
					this.pieceCount[player & 1]--;
				}

				return --this.pieceCount[player];
			};

			Board.prototype.getPiece = function (row, col) {
				return this.grid[(row << 3) + col];
				//return this.grid[row * this.cols + col];
			};

			Board.prototype.getPlayer = function (row, col) {
				return this.getPiece(row, col) & 3;
			};

			Board.prototype.clear = function () {
				this.player = PlayerOne;
				this.multiTurn = false;
				for (var i = 0; i < BoardArea; i++) {
					this.grid[i] = 0;
				}
			};

			Board.prototype.coordToIndex = function (row, col) {
				return (row << 3) + col;
			};

			Board.prototype.opponent = function () {
				return this.player ^ 1;
			};

			Board.prototype.area = function () {
				return BoardArea;
			};

			Board.prototype.validateMove = function (fromRow, fromCol, toRow, toCol) {
				// Source piece must be the current player.
				if (this.getPlayer(fromRow, fromCol) != this.player) {
					throw "piece is not yours";
				}

				if (this.getPiece(toRow, toCol)) {
					throw "square is not empty";
				}

				// Movement must be diagonal.
				var rowDelta = Math.abs(fromRow - toRow);
				var colDelta = Math.abs(fromCol - toCol);
				if (rowDelta != colDelta) {
					throw "moves must be diagonal";
				}

				// Non-kings cannot move backwards.
				if (!(this.getPiece(fromRow, fromCol) & 4)) {
					if (this.player == PlayerTwo && toRow > fromRow) {
						throw "this piece must move forward";
					}
					else if (this.player == PlayerOne && toRow < fromRow) {
						throw "this piece must move forward";
					}
				}

				if (rowDelta > 1) {
					if (rowDelta > 2) {
						throw "you cannot move more than 2 spaces";
					}
					if (!this.isValidJump(fromRow, fromCol, toRow, toCol)) {
						throw "you can only move 2 spaces by jumping another piece";
					}
					return true;
				}

				// A multi-turn must have ended in a jump.
				if (this.multiTurn) {
					throw "if a jump is possible, you must jump";
				}

				// Detect if any jumps are possible.
				for (var row = 0; row < this.rows; row++) {
					for (var col = 0; col < this.cols; col++) {
						if (this.getPlayer(row, col) != this.player) {
							continue;
						}
						var isKing = this.isKing(row, col);
						if (this.canCapture(isKing, row, col)) {
							throw "if a jump is possible, you must jump";
						}
					}
				}
			};

			// Move lists are vectors where each entry is a move, encoded as a
			// bitstring for efficiency.
			Board.prototype.fillMoveList = function (moveList) {
				var start = BoardArea + (this.player & 1) * PiecesPerPlayer;
				var canCapture = false;
				for (var i = 0; i < PiecesPerPlayer; i++) {
					var piece = this.grid[start + i];
					if (piece == -1) {
						continue;
					}

					var fromRow = piece >> 8;
					var fromCol = piece & 0xFF;
					var isKing = this.grid[this.coordToIndex(fromRow, fromCol)] & 4;
					var opponent = this.player ^ 1;

					// Test movement from row 0 -> E.
					if ((isKing || this.player == 2) && fromRow != this.rows - 1) {
						var left = 0, right = 0;
						if (fromCol != 0) {
							left = this.getPiece(fromRow + 1, fromCol - 1);
							if (!left && !canCapture) {
								moveList.add(fromRow, fromCol, fromRow + 1, fromCol - 1);
							}
						}
						if (fromCol != this.cols - 1) {
							right = this.getPiece(fromRow + 1, fromCol + 1);
							if (!right && !canCapture) {
								moveList.add(fromRow, fromCol, fromRow + 1, fromCol + 1);
							}
						}

						// Test if we can add any jumps.
						if (fromRow != this.rows - 2) {
							if ((left & 3) == opponent &&
									fromCol >= 2 &&
									!this.getPiece(fromRow + 2, fromCol - 2)) {
								if (!canCapture) {
									moveList.clear();
									canCapture = true;
								}
								moveList.add(fromRow, fromCol, fromRow + 2, fromCol - 2);
							}
							if ((right & 3) == opponent &&
									fromCol < this.cols - 2 &&
									!this.getPiece(fromRow + 2, fromCol + 2)) {
								if (!canCapture) {
									moveList.clear();
									canCapture = true;
								}
								moveList.add(fromRow, fromCol, fromRow + 2, fromCol + 2);
							}
						}
					}

					// Test movement from row E -> 0.
					if ((isKing || this.player == 3) && fromRow != 0) {
						var left = 0, right = 0;
						if (fromCol != 0) {
							left = this.getPiece(fromRow - 1, fromCol - 1);
							if (!left && !canCapture) {
								moveList.add(fromRow, fromCol, fromRow - 1, fromCol - 1);
							}
						}
						if (fromCol != this.cols - 1) {
							right = this.getPiece(fromRow - 1, fromCol + 1);
							if (!right && !canCapture) {
								moveList.add(fromRow, fromCol, fromRow - 1, fromCol + 1);
							}
						}

						// Test if we can add any jumps.
						if (fromRow >= 2) {
							if ((left & 3) == opponent &&
									fromCol >= 2 &&
									!this.getPiece(fromRow - 2, fromCol - 2)) {
								if (!canCapture) {
									moveList.clear();
									canCapture = true;
								}
								moveList.add(fromRow, fromCol, fromRow - 2, fromCol - 2);
							}
							if ((right & 3) == opponent &&
									fromCol < this.cols - 2 &&
									!this.getPiece(fromRow - 2, fromCol + 2)) {
								if (!canCapture) {
									moveList.clear();
									canCapture = true;
								}
								moveList.add(fromRow, fromCol, fromRow - 2, fromCol + 2);
							}
						}
					}
				}
			};

			Board.prototype.isValidJump = function (fromRow, fromCol, toRow, toCol) {
				// The intervening empty square must have an enemy piece.
				var middleRow = (fromRow + toRow) >> 1;
				var middleCol = (fromCol + toCol) >> 1;
				var middlePiece = this.getPlayer(middleRow, middleCol);
				return middlePiece == (this.player ^ 1);
			}

			Board.prototype.isValidHop = function (fromRow, fromCol, toRow, toCol) {
				// Destination must be empty.
				if (this.getPiece(toRow, toCol)) {
					return false;
				}
				return this.isValidJump(fromRow, fromCol, toRow, toCol);
			};

			Board.prototype.canCapture = function (isKing, row, col) {
				return (row >= 2 && col >= 2 &&
						(isKing || this.player == 3) &&
						this.isValidHop(row, col, row - 2, col - 2)) ||
						(row >= 2 && col < this.cols - 2 &&
								(isKing || this.player == 3) &&
								this.isValidHop(row, col, row - 2, col + 2)) ||
						(row < this.rows - 2 && col >= 2 &&
								(isKing || this.player == 2) &&
								this.isValidHop(row, col, row + 2, col - 2)) ||
						(row < this.rows - 2 && col < this.cols - 2 &&
								(isKing || this.player == 2) &&
								this.isValidHop(row, col, row + 2, col + 2));
			};

			Board.prototype.isKing = function (row, col) {
				return this.getPiece(row, col) & 4;
			};

			Board.prototype.evaluateScore = function (player) {
				return (this.pieceCount[player & 1] * 5) +
						(this.pieceCount[player]);
			};

			Board.prototype.move = function (fromRow, fromCol, toRow, toCol) {
				var fromIndex = this.coordToIndex(fromRow, fromCol);
				var piece = this.grid[fromIndex];
				this.grid[fromIndex] = 0;

				if ((this.player == 2 && toRow == this.rows - 1) ||
						(this.player == 3 && toRow == 0)) {
					// King me!
					piece |= 4;
					this.pieceCount[this.player & 1]++;
				}

				// Update board.
				var toIndex = this.coordToIndex(toRow, toCol);
				this.grid[toIndex] = piece;

				// Update tracking.
				var key = (piece >> 8) & 0xFF;
				this.grid[BoardArea + key] = (toRow << 8) | toCol;

				// Trick: 1 and -1 have bit 1 set, 2 and -2 do not.
				if (!((fromRow - toRow) & 1)) {
					// We assume that move validity was checked already, and that
					// if this piece has an even delta, it is jumping something.
					var middleRow = (fromRow + toRow) >> 1;
					var middleCol = (fromCol + toCol) >> 1;
					if (!this.removePiece(middleRow, middleCol)) {
						return true;
					}

					// A successful jump means the player gets to jump again.
					// Detect that by seeing if any two-space hops are legal.
					var multiTurn = this.canCapture(piece & 4, toRow, toCol);
					this.multiTurn = multiTurn;
					if (multiTurn) {
						return false;
					}
				}

				// Switch to the next player.
				this.player ^= 1;

				return false;
			};

			Board.prototype.copy = function () {
				var board = new Board();
				var gridSize = this.grid.length;
				var to = board.grid;
				var from = this.grid;
				for (var i = 0; i < gridSize; i++) {
					to[i] = from[i];
				}
				board.player = this.player;
				board.multiTurn = this.multiTurn;
				board.pieceCount[0] = this.pieceCount[0];
				board.pieceCount[1] = this.pieceCount[1];
				board.pieceCount[2] = this.pieceCount[2];
				board.pieceCount[3] = this.pieceCount[3];
				return board;
			};

			// If no moves are available, returns that the opponent won.
			// If the move is a winning move, return the current player.
			// Otherwise, return 0.
			Board.prototype.moveRandom = function (moves) {
				moves.clear();
				this.fillMoveList(moves);
				if (!moves.length) {
					return this.opponent();
				}
				var r = Math.floor(Math.random() * moves.length);
				var move = moves.list[r];
				var fromRow = move >> 24;
				var fromCol = (move >> 16) & 0xFF;
				var toRow = (move >> 8) & 0xFF;
				var toCol = (move) & 0xFF;
				if (this.move(fromRow, fromCol, toRow, toCol)) {
					return this.player;
				}
				return 0;
			};

			Board.prototype.init = function () {
				this.clear();

				// First fill in player one.
				var odds = 1;
				var key = 0;
				for (var i = 0; i < RowsPerPlayer; i++) {
					for (var j = odds; j < this.cols; j += 2) {
						this.initPiece(i, j, key, PlayerOne);
						key++;
					}
					odds ^= 1;
				}

				// Fill in player two.
				odds = 0;
				for (var i = RowsPerPlayer + 2; i < this.rows; i++) {
					for (var j = odds; j < this.cols; j += 2) {
						this.initPiece(i, j, key, PlayerTwo);
						key++;
					}
					odds ^= 1;
				}
			};

			this.player = function () {
				return this.board.player;
			};
			this.playerAt = function (row, col) {
				return this.board.getPlayer(row, col);
			};
			this.inMultiTurn = function () {
				return this.board.multiTurn;
			};

			this.move = function (fromRow, fromCol, toRow, toCol) {
				this.board.validateMove(fromRow, fromCol, toRow, toCol);
				if (this.board.move(fromRow, fromCol, toRow, toCol)) {
					return this.board.player;
				}
				var moves = new Checkers.MoveList(BoardArea);
				this.board.fillMoveList(moves);
				if (!moves.length) {
					return this.board.opponent();
				}
				return 0;
			};

			this.suggest = function (maxTime) {
				return UCT(this.board, maxTime);
			};

			this.board = new Board();
			this.board.init();

			this.start = function () {
				ui.start(this);
			};
		}
	};

	function CanvasCheckers(rows, cols) {
		this.rows = rows || 8;
		this.cols = cols || 8;
		var computer = Checkers.PlayerTwo;

		function Draw(b, selRow, selCol) {
			// Now, fill in player pieces.
			var boardChips = [];
			for (var row = 0; row < rows; row++) {
				for (var col = 0; col < cols; col++) {
					var piece = b.getPlayer(row, col);
					if (b.isKing(row, col)) {
						piece += 2;
					}
					boardChips.push(piece);
				}
			}
			reloadBoard(boardChips);
			selectChip(selRow, selCol);
		}

		var game;
		var selectedRow = undefined;
		var selectedCol = undefined;

		var computerPlay = function () {
			var moves = new Checkers.MoveList(game.board.area());
			game.board.fillMoveList(moves);

			var result;
			if (moves.length > 1) {
				result = game.suggest(500);
			} else {
				move = moves.list[0];
				result = {};
				result.fromRow = move >> 24;
				result.fromCol = (move >> 16) & 0xFF;
				result.toRow = (move >> 8) & 0xFF;
				result.toCol = (move) & 0xFF;
			}

			var winner = game.move(result.fromRow, result.fromCol,
					result.toRow, result.toCol);
			if (winner) {
				var color = winner == Checkers.PlayerOne
						? "White"
						: "Black";
				utils.showAlert(color + " player wins! Refresh to play again.");
				return;
			}

			Draw(game.board);
			if (game.inMultiTurn()) {
				setTimeout(computerPlay, 200);
			}
		};

		var onClick = function (evt) {

			var node = evt.target;
			var row = parseInt(node.getAttribute('y'));
			var col = parseInt(node.getAttribute('x'));

			// Don't let the player move during the computer's turn.
			if (game.player() == computer) {
				return;
			}

			var player = game.playerAt(row, col);
			if (player) {
				if (player != game.player()) {
					return;
				}

				// If the player is selecting an own piece, just update the display.
				if (!game.inMultiTurn()) {
					selectedRow = row;
					selectedCol = col;
					Draw(game.board, selectedRow, selectedCol);
					return;
				}
			}

			// If the player is selecting an empty square without having done
			// anything yet, just return.
			if (selectedRow === undefined) {
				return;
			}

			var fromRow = selectedRow;
			var fromCol = selectedCol;
			if (!game.inMultiTurn()) {
				selectedRow = undefined;
				selectedCol = undefined;
			}

			// Try to move. If it's not valid, redraw.
			try {
				var winner = game.move(fromRow, fromCol, row, col);
				if (winner) {
					var color = winner == Checkers.PlayerOne
							? "Red"
							: "Blue";
					utils.showAlert(color + " player wins! Refresh to play again.");
					return;
				}
			} catch (e) {
				utils.showAlert('Invalid move: ' + e);
				Draw(game.board, selectedRow, selectedCol);
				return;
			}

			// The move was valid. If the player is allowed to move again,
			// re-select the piece and redraw.
			if (game.inMultiTurn()) {
				selectedRow = row;
				selectedCol = col;
				Draw(game.board, row, col);
				return;
			}

			Draw(game.board);

			// Otherwise, draw and let the computer make a move.
			if (computer) {
				setTimeout(computerPlay, 200);
			}
		};

		this.switchSides = function () {
			computer = computer ^ 1;
			setTimeout(computerPlay, 250);
		};

		this.start = function (game_) {
			game = game_;
			checker.table.addEventListener('click', onClick);
			Draw(game.board);
		};
	}


	function UCTNode(move, player) {
		this.visits = 1;
		this.score = 0;
		this.children = null;
		this.player = player;
		this.move = move;
	}

	UCTNode.prototype.opponent = function () {
		return this.player ^ 1;
	};

	UCTNode.prototype.append = function (child) {
		if (!this.children)
			this.children = [child];
		else
			this.children.push(child);
	};

	UCTNode.prototype.playFor = function (board) {
		var move = this.move;
		var fromRow = move >> 24;
		var fromCol = (move >> 16) & 0xFF;
		var toRow = (move >> 8) & 0xFF;
		var toCol = (move) & 0xFF;
		return board.move(fromRow, fromCol, toRow, toCol);
	};

	UCTNode.prototype.expand = function (board, moveList) {
		moveList.clear();
		board.fillMoveList(moveList);

		for (var i = 0; i < moveList.length; i++) {
			var child = new UCTNode(moveList.list[i], board.player);
			this.append(child);
		}
	};

	UCTNode.prototype.ucb = function (coeff) {
		var score = (this.score / this.visits);
		return score + Math.sqrt(coeff / this.visits);
	};

	UCTNode.prototype.findBestChild = function () {
		var coeff = 20 * Math.log(this.visits);
		var bestScore = -Infinity;
		var bestChild = null;
		for (var i = 0; i < this.children.length; i++) {
			var child = this.children[i];
			var score = child.ucb(coeff);
			if (score > bestScore) {
				bestScore = score;
				bestChild = child;
			}
		}
		return bestChild;
	};

	function UCT(original, maxTime) {
		var root = new UCTNode(0, 0);
		var moveList = new Checkers.MoveList(original.area());
		var MaturityThreshold = 200;

		// Populate the first node.
		root.expand(original, moveList);
		var history = [root];

		function playout(board) {
			var nmoves = 0;

			while (++nmoves < 60) {
				var result = board.moveRandom(moveList);
				if (result) {
					return result;
				}
			}
			var p1 = board.evaluateScore(Checkers.PlayerOne);
			var p2 = board.evaluateScore(Checkers.PlayerTwo);
			if (p1 > p2) {
				return Checkers.PlayerOne;
			}
			if (p1 < p2) {
				return Checkers.PlayerTwo;
			}
			return 0;
		}

		function run() {
			var depth = 1;
			var board = original.copy();
			var node = root;
			var winner = 0;

			while (true) {
				if (node.children === null) {
					if (node.visits >= MaturityThreshold) {
						node.expand(board, moveList);

						// Leaf node - go directly to update.
						if (node.children === null) {
							winner = node.opponent();
							history[depth++] = node;
							break;
						}
						continue;
					}
					winner = playout(board);
					break;
				}
				node = node.findBestChild();
				history[depth++] = node;
				if (node.playFor(board)) {
					winner = board.player;
					break;
				}
			}

			for (var i = 0; i < depth; i++) {
				node = history[i];
				node.visits++;
				if (winner == node.player) {
					node.score += 1;
				}
				else if (winner != 0) {
					node.score -= 1;
				}
			}
		}

		var TotalPlayouts = 0;
		var start = Date.now();
		var elapsed;
		while (true) {
			for (var i = 0; i < 500; i++) {
				run();
			}
			TotalPlayouts += 500;
			elapsed = Date.now() - start;
			if (elapsed >= maxTime) {
				break;
			}
		}

		var bestChild = null;
		var bestScore = -Infinity;
		for (var i = 0; i < root.children.length; i++) {
			var child = root.children[i];
			if (child.visits > bestScore) {
				bestChild = child;
				bestScore = child.visits;
			}
		}

		var move = bestChild.move;
		var fromRow = move >> 24;
		var fromCol = (move >> 16) & 0xFF;
		var toRow = (move >> 8) & 0xFF;
		var toCol = (move) & 0xFF;

		return {
			fromRow: fromRow,
			fromCol: fromCol,
			toRow: toRow,
			toCol: toCol
		};
	}

	(function() {
		var ui = new CanvasCheckers(8, 8);
		var game = new Checkers.Game(ui);
		game.start();
	})();

});
