loader.addOnLoad(function() {

	var checker = {
		table: null
	};

	utils.mixin(PARAMS, {
		MAX_BOARD_HEIGHT: 100
	});

	var REVERSI = {
		EMPTY_RECT: 0,
		WHITE_CHIP: 1,
		BLACK_CHIP: 2,
		PERMITTED_POSITION: 3,
		getBoardChipClass : function() {
			return "";
		},
		generateBoardIsCompleted : function(boardNode) {
			boardNode.addEventListener('click', function(evt) {
				var node = evt.target;
				if (node && node.tagName == "DIV" && utils.hasClass(node, "PermittedPosition")) {
					REVERSI.onClickPermittedPosition(node);
				}
			});
		},
		onClickPermittedPosition: function(node) {
			utils.mobileClick(node, function() {
				var index = node.getAttribute("index");
				if (node && index) {
					Clk(index);
				}
			}, 100);
		}
	};

	var PLAYER = REVERSI;

	// generate html Board
	// Checker and Reversi
	function generateStringHTMLCheckerBoard(node) {
		var result = [];
		result.push("<div id='boardTable' class='boardContainer'>");
		var index = 0;
		for (var i = 0; i < 8; ++i) {
			result.push("<div class='boardRow'>");
			for (var j = 0; j < 8; ++j) {
				var _c = PLAYER.getBoardChipClass(i, j);
				result.push("<div index='" + index + "' id='" + generateChipId(index) + "' y='" + i + "' x='" + j + "' class='" + _c + "'></div>");
				index += 1;
			}
			result.push("</div>");
		}
		result.push("</div>");
		node.innerHTML = result.join("");
		var table = $('boardTable');
		PLAYER.generateBoardIsCompleted(table);
		checker.table = table;
	}

	function generateChipId(index) {
		return (["chip", index].join("_"));
	}

	function onChangeBoardSize(w, h) {
		var node = checker.table;
		var setH = Math.min(w, h) - 10;
		if (setH < PARAMS.MAX_BOARD_HEIGHT) {
			setH = PARAMS.MAX_BOARD_HEIGHT
		}
		node.style.height = setH + 'px';
		node.style.width = setH + 'px';
		var tdH = setH / 8 - 3;
		var tds = $$(node, "div");
		for (var i = 0, l = tds.length; i < l; ++i) {
			var td = tds[i];
			if (td.getAttribute("x")) {
				td.style.width = tdH + 1 + "px";
				td.style.height = tdH + 1 + "px";
				td.style.backgroundSize = tdH + "px";
			}
		}
	}

	function resizeBoard() {
		var node = $("board");
		onChangeBoardSize(node.clientWidth, node.clientHeight)
	}

	function placeChipsToBoard(boardChips, moveSide) {
		for (var i = 0; i < boardChips.length; ++i) {
			var chipState = boardChips[i];
			var chipNode = $("chip_" + i);
			var className = "";
			switch (chipState) {
				case PLAYER.WHITE_CHIP :
					className = "whiteChip";
					break;
				case PLAYER.BLACK_CHIP :
					className = "blackChip";
					break;
				case PLAYER.PERMITTED_POSITION:
					if (moveSide == "0") {
						className = "PermittedPosition";
					}
					break;
			}
			utils.addClass(chipNode, className);
		}
	}

	function removeClasses(className) {
		var boardNode = $('boardTable');
		var nodes = boardNode.getElementsByClassName(className);
		while (nodes.length > 0) {
			utils.removeClass(nodes[0], className);
		}
	}

	function reloadBoard(boardChips, moveSide) {
		removeClasses("PermittedPosition");
		removeClasses("blackChip");
		removeClasses("whiteChip");
		placeChipsToBoard(boardChips, moveSide);
	}

	window.addEventListener("resize", resizeBoard);
	window.addEventListener("orientationchange", resizeBoard);

	(function() {
		var boardNode = $("boardTableContainer");
		generateStringHTMLCheckerBoard(boardNode);
		resizeBoard();
	})();


	// Algorithm from http://www.softholm.com/igra/othello_game.html
	var Empty = 0;
	var Pl1 = 1;
	var Pl2 = 2;
	var Field = new Array(64);

	function Matrix(i, j) {
		return (((i - 1) * 8) + j - 1);
	}

	function NotPlayer(Pl) {
		if (Pl == Pl1) {
			return Pl2;
		}
		else {
			return Pl1;
		}
	}

	function InitField() {
		for (var i = 0; i < 64; i++) {
			Field[i] = Empty;
		}
	}

	function GetNum(Player) {
		var i,N = 0;
		for (i = 0; i < 64; i++) {
			if (Field[i] == Player) {
				N++;
			}
		}
		return N;
	}

	function ComputerPlace(Player) {
		var PlaceTable = new Array(1, 8, 2, 2, 2, 2, 8, 1, 8, 8, 6, 5, 5, 6, 8, 8, 2, 6, 4, 3, 3, 4, 6, 2, 2, 5, 3, 1, 1, 3, 5, 2, 2, 5, 3, 1, 1, 3, 5, 2, 2, 6, 4, 3, 3, 4, 6, 2, 8, 8, 6, 5, 5, 6, 8, 8, 1, 8, 2, 2, 2, 2, 8, 1);
		var PlX = new Array(61);
		var PlY = new Array(61);
		var NPl = 0,I,J,MaxE = 0,E,MinPr = 9;
		for (I = 1; I <= 8; I++) {
			for (J = 1; J <= 8; J++) {
				E = Place(I, J, Player, false);
				if ((MinPr > PlaceTable[Matrix(I, J)]) && (E < 0xFF) && (E > 0)) {
					MinPr = PlaceTable[Matrix(I, J)];
					NPl = 0;
					MaxE = 0;
				}
				if ((E < 0xFF) && (MinPr == PlaceTable[Matrix(I, J)])) {
					if (E > MaxE) {
						MaxE = E;
						NPl = 1;
						PlX[1] = I;
						PlY[1] = J;
					}
					else {
						if (E == MaxE) {
							NPl++;
							PlX[NPl] = I;
							PlY[NPl] = J;
						}
					}
				}
			}
		}
		E = (Math.floor(Math.random() * NPl)) + 1;
		Place(PlX[E], PlY[E], Player, true);
		return ((PlX[E] << 4) | PlY[E]);
	}

	function CanPlace(Player) {
		var I,J,E = 0,E1;
		for (I = 1; I <= 8; I++) {
			for (J = 1; J <= 8; J++) {
				E1 = Place(I, J, Player, false);
				if (E1 < 0xFF) {
					E += E1;
				}
			}
		}
		return (E > 0);
	}

	function Place(x, y, Player, Placed)
		/*0..64 - Ok (Съедено)
		 $FF - not empty*/ {
		var I,J,x1,y1,Eated,E;
		if (Field[Matrix(x, y)] != Empty) {
			return (0xFF);
		}
		else {
			Eated = 0;
			for (I = -1; I <= 1; I++) {
				for (J = -1; J <= 1; J++) {
					E = 0;
					x1 = x + I;
					y1 = y + J;
					while (((x1 >= 1) && (x1 <= 8) && (y1 >= 1) && (y1 <= 8)) && (Field[Matrix(x1, y1)] != Empty) && (Field[Matrix(x1, y1)] != Player)) {
						E++;
						x1 = x1 + I;
						y1 = y1 + J;
					}
					if ((Placed) && ((x1 >= 1) && (x1 <= 8) && (y1 >= 1) && (y1 <= 8)) && (Field[Matrix(x1, y1)] != Empty) && (E > 0)) {
						Field[Matrix(x, y)] = Player;
						x1 = x + I;
						y1 = y + J;
						while (((x1 >= 1) && (x1 <= 8) && (y1 >= 1) && (y1 <= 8)) && (Field[Matrix(x1, y1)] != Empty) && (Field[Matrix(x1, y1)] != Player)) {
							Field[Matrix(x1, y1)] = Player;
							x1 = x1 + I;
							y1 = y1 + J;
						}
					}
					if (((x1 >= 1) && (x1 <= 8) && (y1 >= 1) && (y1 <= 8)) && (Field[Matrix(x1, y1)] != Empty))
						Eated += E;
				}
			}
			return (Eated);
		}
	}

	var CurrPlayer,Computer,InGame = false,NumPl1,NumPl2;

	function Result() {
		if (NumPl1 > NumPl2) {
			utils.showAlert('White is WIN !');
		}
		else {
			if (NumPl1 < NumPl2) {
				utils.showAlert('Black is WIN !');
			}
			else {
				utils.showAlert('DRAW !');
			}
		}
	}

	function InitMainWindow() {
		Computer = Pl2;
		NumPl1 = GetNum(Pl1);
		NumPl2 = GetNum(Pl2);
		CurrPlayer = Pl1;
		InGame = true;
	}

	function NewGame() {
		var i;
		InitField();
		redraw();
		for (i = 0; i < 4; i++) {
			Field[Matrix(i + 1, i + 1)] = Pl1;
			Field[Matrix(8 - i, 8 - i)] = Pl1;
			Field[Matrix(i + 1, 8 - i)] = Pl2;
			Field[Matrix(8 - i, i + 1)] = Pl2;
			redraw();
		}
		for (i = 0; i < 3; i++) {
			Field[Matrix(i + 1, i + 1)] = Empty;
			Field[Matrix(8 - i, 8 - i)] = Empty;
			Field[Matrix(i + 1, 8 - i)] = Empty;
			Field[Matrix(8 - i, i + 1)] = Empty;
			redraw();
		}
		CurrPlayer = Pl1;
		NumPl1 = 2;
		NumPl2 = 2;
		InGame = true;
		redraw();
	}

	function Clk(n) {
		var Placed;
		if ((CurrPlayer == Computer) || (! InGame) || (CurrPlayer == 0xFF)) {
		}
		else {
			Placed = Place((Math.floor(n / 8)) + 1, (n % 8) + 1, CurrPlayer, false);
			if (Placed == 0) {
				utils.showAlert("You can't put on this place!");
			}
			else {
				if (Placed == 0xFF) {
					utils.showAlert("This place isn't empty");
				}
				else {
					Placed = Place((Math.floor(n / 8)) + 1, (n % 8) + 1, CurrPlayer, true);
					CurrPlayer = NotPlayer(CurrPlayer);
					NumPl1 = GetNum(Pl1);
					NumPl2 = GetNum(Pl2);
					redraw();
					if ((! CanPlace(Pl1)) && (! CanPlace(Pl2))) {
						InGame = false;
						Result();
					}
					if (! CanPlace(CurrPlayer)) {
						CurrPlayer = NotPlayer(CurrPlayer);
					}
				}
			}
		}
	}

	function Timer() {
		var PPP,IDTimer;
		if ((InGame) & (CurrPlayer == Computer)) {
			if (CanPlace(CurrPlayer)) {
				PPP = ComputerPlace(CurrPlayer);
				//     Place(PPP >> 4,PPP & 0x0F,CurrPlayer,true);
				NumPl1 = GetNum(Pl1);
				NumPl2 = GetNum(Pl2);
				redraw();
			}
			CurrPlayer = NotPlayer(CurrPlayer);
			if ((! CanPlace(Pl1)) && (! CanPlace(Pl2))) {
				InGame = false;
				Result();
			}
			if (! CanPlace(CurrPlayer)) {
				CurrPlayer = NotPlayer(CurrPlayer);
			}
		}
		IDTimer = setTimeout(Timer, 400);
		redrowPermitted();
	}

	function redraw() {
		var i;
		reloadBoard(Field);
		redrowPermitted();
		if (NumPl1 > NumPl2) {
			i = 10 - Math.floor((NumPl1 * 10) / (NumPl1 + NumPl2));
		}
		if (NumPl1 < NumPl2) {
			i = 10 + Math.floor((NumPl2 * 10) / (NumPl1 + NumPl2));
		}
		if (NumPl1 == NumPl2) {
			i = 10;
		}
	}

	function redrowPermitted() {
		removeClasses("PermittedPosition");
		for (var i = 0; i < 64 && CurrPlayer == Pl1; i++) {
			var Placed = Place((Math.floor(i / 8)) + 1, (i % 8) + 1, CurrPlayer, false);
			if (Placed != 0 && Placed != 0xFF) {
				var node = $(generateChipId(i));
				utils.addClass(node, "PermittedPosition");
			}
		}
	}

	(function() {
		InitMainWindow();
		NewGame();
		setTimeout(Timer, 1000);
	})();
});
